/*
 * WYMeditor : what you see is What You Mean web-based editor
 * Copyright (c) 2008 Jean-Francois Hovinne, http://www.wymeditor.org/
 * Dual licensed under the MIT (MIT-license.txt)
 * and GPL (GPL-license.txt) licenses.
 *
 * For further information visit:
 *      http://www.wymeditor.org/
 *
 * File Name:
 *      skin.css
 *      main stylesheet for the django WYMeditor skin
 *      See the documentation for more info.
 *
 * File Authors:
 *      Daniel Reszka (d.reszka a-t wymeditor dotorg)
 *      Jannis Leidel (jannis@leidel.info)
*/

/*TRYING TO RESET STYLES THAT MAY INTERFERE WITH WYMEDITOR*/
        .wym_skin_semanticeditor p,
        .wym_skin_semanticeditor h3,
        .wym_skin_semanticeditor ul,
        .wym_skin_semanticeditor li { margin: 0; padding: 0; border-width:0; list-style: none; }

/*HIDDEN BY DEFAULT*/
        .wym_skin_semanticeditor .wym_area_left          { display: block; }
        .wym_skin_semanticeditor .wym_area_right         { display: block; }

/*TYPO*/
        .wym_skin_semanticeditor h2             { }
        .wym_skin_semanticeditor h3             { font-size: 100%; color: black;}
        .wym_skin_semanticeditor li             { font-size: 100%; /* = 10px */}
        .wym_skin_semanticeditor li { margin: 0; padding: 0; border}

/*WYM_BOX*/
        .wym_skin_semanticeditor                { border: 0px; background: none; padding: 0px}

    /*auto-clear the wym_box*/
        .wym_skin_semanticeditor:after 	        { content: "."; display: block; height: 0; clear: both; visibility: hidden; }
 * html .wym_skin_semanticeditor                { height: 1%;}


/*WYM_HTML*/
        .wym_skin_semanticeditor .wym_html               { width: 98%;}
        .wym_skin_semanticeditor .wym_html textarea      { width: 100%; height: 200px; border: 1px solid #ccc; background: white;  }


/*WYM_IFRAME*/
        .wym_skin_semanticeditor .wym_iframe             { width: 98%;}
        .wym_skin_semanticeditor .wym_iframe iframe      { width: 100%; height: 600px; border: 1px solid #ccc; background: white }


/*AREAS*/
        .wym_skin_semanticeditor .wym_area_left          { width: 150px; float: left;}
        .wym_skin_semanticeditor .wym_area_right         { width: 190px; float: right;}
        .wym_skin_semanticeditor .wym_area_bottom        { clear: both; display: none;}
 * html .wym_skin_semanticeditor .wym_area_main          { height: 1%;}
 * html .wym_skin_semanticeditor .wym_area_top           { height: 1%;}
 *+html .wym_skin_semanticeditor .wym_area_top           { height: 1%;}

/*SECTIONS SYSTEM*/

    /*common defaults for all sections*/
        .wym_skin_semanticeditor .wym_section            { margin-bottom: 5px; }
        .wym_skin_semanticeditor .wym_section h2,
        .wym_skin_semanticeditor .wym_section h3         { }
        .wym_skin_semanticeditor .wym_section a {
            display: block;
        }

      /*hide section titles by default*/
        .wym_skin_semanticeditor .wym_section h2         { display: none; }
      /*disable any margin-collapse*/
        .wym_skin_semanticeditor .wym_section            { padding-top: 1px; padding-bottom: 1px; }
      /*auto-clear sections*/
        .wym_skin_semanticeditor .wym_section ul:after   { content: "."; display: block; height: 0; clear: both; visibility: hidden; }
 * html .wym_skin_semanticeditor .wym_section ul         { height: 1%;}

    /*option: add this class to a section to make it render as a panel*/
        .wym_skin_semanticeditor .wym_panel              { }
        .wym_skin_semanticeditor .wym_panel h2           { display: block; text-align: center;}

    /*option: add this class to a section to make it render as a dropdown menu*/
        .wym_skin_semanticeditor .wym_dropdown h2        { display: block; }
        .wym_skin_semanticeditor .wym_dropdown ul        { display: none; position: absolute; background: white; }
        .wym_skin_semanticeditor .wym_dropdown:hover ul,
        .wym_skin_semanticeditor .wym_dropdown.hover ul  { display: block; }

        .wym_skin_semanticeditor .wym_tools ul  { padding-left: 0px !important; margin-left: 0px;}


/*CLASSES*/
        .wym_area_right .wym_panel ul {padding-left: 0;}
        .wym_skin_semanticeditor .wym_classes                { overflow-y: auto;
                                                               overflow-x: visible; }

/*DIALOGS*/
        .wym_dialog div.row         { margin-bottom: 5px;}
        .wym_dialog div.row input   { margin-right: 5px;}
        .wym_dialog div.row label   { float: left; width: 150px; display: block; text-align: right; margin-right: 10px; }
        .wym_dialog div.row-indent  { padding-left: 160px; }
        /*autoclearing*/
        .wym_dialog div.row:after            { content: "."; display: block; height: 0; clear: both; visibility: hidden; }
        .wym_dialog div.row                  { display: inline-block; }
            /* Hides from IE-mac \*/
            * html .wym_dialog div.row       { height: 1%; }
            .wym_dialog div.row              { display: block; }
            /* End hide from IE-mac */

/*WYMEDITOR_LINK*/
        a.wym_wymeditor_link        { text-indent: -9999px; float: right; display: block; width: 50px; height: 15px; background: url(../wymeditor_icon.png);  overflow: hidden; text-decoration: none;  }


.wym_skin_semanticeditor .wym_dropdown {
    padding: 0;
    border: solid gray;
    border-width: 1px 1px 0 1px;
}

.wym_skin_semanticeditor .wym_dropdown ul {
        border: solid gray;
        border-width: 0 1px 1px 1px;
        margin-left: -1px;
        padding: 5px 10px 5px 3px;
}
